# calculate the transformed phenotypic residual V in the null model with related individuals, also calculate the P matrix
# note, V=PY and P=\Sigma_0^{-1}-\Sigma_0^{-1}Z(Z^T\Sigma_0^{-1}Z)^{-1}Z^T\Sigma_0^{-1}
# see Eq 2.1 of Wu et al. 2017, AOAS

calculateV <- function(Y, Z, PHI.pf) # Z includes intercept
{
  nind <- length(Y)
  if (nrow(Z) != nind)
  {
    stop("Dimension of response / covariates in the null regression model is wrong!")
  }
  
  nind.pf <- nrow(PHI.pf)
  nfam <- nind / nind.pf
  s2.hat <- estimateVC(Y, Z, PHI.pf)
  SIGMA.hat.pf <- s2.hat[1] * diag(nind.pf) + s2.hat[2] * PHI.pf
  SIGMA.hat.inv.pf <- solve(SIGMA.hat.pf)
  SIGMA.hat.inv <- kronecker(diag(nfam), SIGMA.hat.inv.pf)
  #beta.hat <- solve(t(Z) %*% SIGMA.hat.inv %*% Z) %*% (t(Z) %*% SIGMA.hat.inv %*% Y)
  #Y.hat <- Z %*% beta.hat
  #V <- SIGMA.hat.inv %*% (Y - Y.hat)
  SinvZ <- SIGMA.hat.inv %*% Z
  SinvP0Sinv <- SIGMA.hat.inv - SinvZ %*% solve(t(Z) %*% SinvZ) %*% t(SinvZ)
  V <- SinvP0Sinv %*% Y
  
  return(list(V = V, SinvP0Sinv = SinvP0Sinv))
}
