# estimate variance components se^2 and sa^2 in the null model
# see Eq 2.1 of Wu et al. 2017, AOAS

estimateVC <- function(Y, Z, PHI.pf) # Z includes intercept
{
  nind <- length(Y)
  if (nrow(Z) != nind)
  {
    stop("Dimension of response / covariates in the null regression model is wrong!")
  }
  nind.pf <- nrow(PHI.pf)
  nfam <- nind / nind.pf
  
  neglogL <- function(par, nfam, nind.pf, y, z, Phi.fi)
  {
    Sigma.fi <- par[1] * diag(nind.pf) + par[2] * Phi.fi
    Sigma.fi.inv <- solve(Sigma.fi)
    Sigma.inv <- kronecker(diag(nfam), Sigma.fi.inv)
    bet.hat <- solve(t(z) %*% Sigma.inv %*% z) %*% (t(z) %*% Sigma.inv %*% y)
    mu.hat <- z %*% bet.hat
    y.mat <- matrix(y, ncol = nfam)
    mu.hat.mat <- matrix(mu.hat, ncol = nfam)
    logL <- 0
    for (i in 1 : nfam)
    {
      #ix.fi <- nind.pf * (i - 1) + (1 : nind.pf)
      #y.fi <- t(y[ix.fi, ])
      y.fi <- y.mat[, i]
      #mu.hat.fi <- t(mu.hat[ix.fi, ])
      mu.hat.fi <- mu.hat.mat[, i]
      logL.fi <- dmvnorm(y.fi, mean = mu.hat.fi, sigma = Sigma.fi, log = T)
      logL <- logL + logL.fi
    }
    
    return(-logL)
  }
  GLS <- nlminb(c(1.9, 2.1), neglogL, lower = 0, upper = 5, nfam = nfam, nind.pf = nind.pf, y = Y, z = Z, Phi.fi = PHI.pf)

  return(c(GLS$par[1], GLS$par[2]))
}
