# generate phenotype data (under Ha) for multiple families, s2 contains two components: se^2 and sa^2

generateZY_Ha <- function(nfam, pedstr, b, s2, PHI.pf, G, g) # Z includes intercept
{
  nind.pf <- nrow(pedstr)
  if ((nrow(PHI.pf) != nind.pf) || (ncol(PHI.pf) != nind.pf))
  {
    stop("# of individuals per family from pedstr and PHI.pf do not match!")
  }
  nind <- nfam * nind.pf
  if (nrow(G) != nind)
  {
    stop("# of individuals in G is wrong!")
  }
  
  # generate Z
  Z <- cbind(1, rnorm(nind))
  # generate Y = Z * b + G * g + e, where e ~ MVN(0, sE2 * I + sA2 * PHI.pf)
  mu <- Z %*% matrix(b, ncol = 1) + G %*% g
  SIGMA.pf <- s2[1] * diag(nind.pf) + s2[2] * PHI.pf
  e <- matrix(t(rmvnorm(nfam, sigma = SIGMA.pf)), ncol = 1)
  Y <- mu + e
  
  return(list(Z = Z, Y = Y))
}
